/*
 *  Author: Anders Blenstrup-pedersen
 * 
 *  Big thanks to mister.t
 *  https://extensions.gnome.org/accounts/profile/mister.t
 *  For a lot of the provided code used in this extension
 *
 *  This extenstion is meant for Gentoo Users
 *  It adds the support for managing:
 *  Apache 2
 *  Redis
 *  Mysql
 *  Solr 4
 *  Solr 5
 *
 *  Since these are the different systems i use for work
 *  I chose to support these servers.
 *  If you are missing the service scripts for Systemd for 
 *  Solr and redis please look in the template folder in the 
 *  Module for an example of how they could look.
 *  Please be aware that they are a copy of my personal
 *  Service files for my local dev system.
 *
 *  This file contains the main extension system.
 */


// extension root object
const Me = imports.misc.extensionUtils.getCurrentExtension();

// import internal modules
const _config = Me.imports._config;

// aliases for used modules
const St = imports.gi.St;
const Lang = imports.lang;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const PanelMenu = imports.ui.panelMenu;
const GLib = imports.gi.GLib;
const Util = imports.misc.util;

let numbersOfTryToActivateMysql = 0;
let numbersOfTryToDesactivateMysql = 0;

let numbersOfTryToActivateApache = 0;
let numbersOfTryToDesactivateApache = 0;

let numbersOfTryToActivateRedis = 0;
let numbersOfTryToDesactivateRedis = 0;

let numbersOfTryToActivateSolr4 = 0;
let numbersOfTryToDesactivateSolr4 = 0;

let numbersOfTryToActivateSolr5 = 0;
let numbersOfTryToDesactivateSolr5 = 0;

let numbersOfPkexecProcess = 0;

let maxNumbersOfTry = 8;

// StatusIcon manager
let statusIcon;
let armsStatusIcon = ['arms-0-service','arms-1-service','arms-2-service','arms-3-service','arms-4-service','arms-5-service'];
let armsLoading = "arms-loading";

// My PopupSwitchMenu
let menuItemApache;
let menuItemMysql;
let menuItemRedis;
let menuItemSolr4;
let menuItemSolr5;



/*
 * Indicator class.
 *
 * Creates an actor in the StatusArea panel. Provides menu for manipulating
 * visiblity of other icons.
 */
const Indicator = new Lang.Class({
    Name: 'Indicator',
    Extends: PanelMenu.Button,

    /**
     * Creates an actor object, which can be added to the status area,
     *
     * @constructor
     * @this {Indicator}
     * @param {string} icon an icon name
     */
    _init: function(icon) {
        let activeServices = 0
        let icon_classname = armsStatusIcon[activeServices];

        if (isApacheActive()) {
            activeServices++;
        }
        if (isMysqlActive()) {
            activeServices++;
        }
        if (isRedisActive()) {
            activeServices++;
        }
        if (isSolr4Active()) {
            activeServices++;
        }
        if (isSolr5Active()) {
            activeServices++;
        }

        icon_classname = armsStatusIcon[activeServices];

        this.parent(0.0, _config.EXTENSION_NAME);

        statusIcon = new St.Icon({
            icon_name: icon,
            style_class: icon_classname
        });

        this.actor.add_actor(statusIcon);

        //this._settings = Convenience.getSettings();
        this._createMenu();
    },

    /**
     * Creates menu for the Indicator. It will be popuped on RMB click.
     *
     * @private
     * @this {Indicator}
     */
    _createMenu: function() {
        menuItemApache = new PopupMenu.PopupSwitchMenuItem("Apache", isApacheActive());
        this.menu.addMenuItem(menuItemApache);
        menuItemApache.statusAreaKey = "Apache";

        menuItemApache.connect('toggled', toggleApacheService);

        menuItemMysql = new PopupMenu.PopupSwitchMenuItem("MySQL", isMysqlActive());
        this.menu.addMenuItem(menuItemMysql);
        menuItemMysql.statusAreaKey = "MySQL";

        menuItemMysql.connect('toggled', toggleMysqlService);

        menuItemRedis = new PopupMenu.PopupSwitchMenuItem("Redis", isRedisActive());
        this.menu.addMenuItem(menuItemRedis);
        menuItemRedis.statusAreaKey = "Redis";

        menuItemRedis.connect('toggled', toggleRedisService);

        menuItemSolr4 = new PopupMenu.PopupSwitchMenuItem("Solr 4", isSolr4Active());
        this.menu.addMenuItem(menuItemSolr4);
        menuItemSolr4.statusAreaKey = "Solr 4";

        menuItemSolr4.connect('toggled', toggleSolr4Service);

        menuItemSolr5 = new PopupMenu.PopupSwitchMenuItem("Solr 5", isSolr5Active());
        this.menu.addMenuItem(menuItemSolr5);
        menuItemSolr5.statusAreaKey = "Solr 5";

        menuItemSolr5.connect('toggled', toggleSolr5Service);
    },

});


/*
 * Extension definition.
 */

function Extension() {
    this._init();
}

Extension.prototype = {
    _init: function() {
        this._indicator = null;
    },

    enable: function() {
        this._indicator = new Indicator('');
        Main.panel.addToStatusArea(_config.EXTENSION_NAME, this._indicator);
    },

    disable: function() {
        this._indicator.destroy();
        this._indicator = null;
    },

};


/**
 * Entry point.
 *
 * Should return an object with callable `enable` and `disable` properties.
 */

// A JSON Object that keeps strings -
//Useful for creating settings


function init() {
    return new Extension();
}



function isApacheActive() {
    // Get current status of apache service
    let [resApache, outApache] = GLib.spawn_command_line_sync("systemctl is-active "+_config.SERVICENAME_APACHE);
    let outApacheString = outApache.toString().replace(/(\r\n|\n|\r)/gm,"");
    return outApacheString == "active";
}

function isMysqlActive() {
    // Get current status of mysql service
    let [resMysql, outMysql] = GLib.spawn_command_line_sync("systemctl is-active "+_config.SERVICENAME_MYSQL);
    let outMysqlString = outMysql.toString().replace(/(\r\n|\n|\r)/gm,"");
    return outMysqlString == "active";
}

function isRedisActive() {
    // Get current status of mysql service
    let [resRedis, outRedis] = GLib.spawn_command_line_sync("systemctl is-active "+_config.SERVICENAME_REDIS);
    let outRedisString = outRedis.toString().replace(/(\r\n|\n|\r)/gm,"");
    return outRedisString == "active";
}

function isSolr4Active() {
    // Get current status of mysql service
    let [resSolr4, outSolr4] = GLib.spawn_command_line_sync("systemctl is-active "+_config.SERVICENAME_SOLR4);
    let outSolr4String = outSolr4.toString().replace(/(\r\n|\n|\r)/gm,"");
    return outSolr4String == "active";
}

function isSolr5Active() {
    // Get current status of mysql service
    let [resSolr5, outSolr5] = GLib.spawn_command_line_sync("systemctl is-active "+_config.SERVICENAME_SOLR5);
    let outSolr5String = outSolr5.toString().replace(/(\r\n|\n|\r)/gm,"");
    return outSolr5String == "active";
}

function toggleApacheService() {
    let action = "start";
    if (isApacheActive()) {
        action = "stop";
    }
    numbersOfPkexecProcess = getNumbersOfPkexecProcess();

    let cmd = _config.PKEXEC_PATH + ' systemctl '+action+' '+_config.SERVICENAME_APACHE;
    if (numbersOfTryToActivateApache == 0 && numbersOfTryToDesactivateApache == 0) {
	    try {
            Util.trySpawnCommandLine(cmd);
            statusIcon.set_property("style_class", armsLoading);
            if (action == "start") {
                GLib.timeout_add(0,300,tryActivateApacheService);
            } else {
                GLib.timeout_add(0,300,tryDesactivateApacheService);
            } 
	    } catch(Exception) {
		  Main.notify("Crash !"+Exception);
	    }
	}
}

function toggleMysqlService() {
    let action = "start";
    if (isMysqlActive()) {
        action = "stop";
    }
    numbersOfPkexecProcess = getNumbersOfPkexecProcess();

    let cmd = _config.PKEXEC_PATH + ' systemctl '+action+' '+_config.SERVICENAME_MYSQL;
    if (numbersOfTryToActivateMysql == 0 && numbersOfTryToDesactivateMysql == 0) {
        try {
            Util.trySpawnCommandLine(cmd);
            statusIcon.set_property("style_class", armsLoading);
            if (action == "start") {
                GLib.timeout_add(0,300,tryActivateMysqlService);
            } else {
                GLib.timeout_add(0,300,tryDesactivateMysqlService);
            }
        } catch(Exception) {
            Main.notify("Big problem :/"+Exception);
        }
    }
}

function toggleRedisService() {
    let action = "start";
    if (isRedisActive()) {
        action = "stop";
    }
    numbersOfPkexecProcess = getNumbersOfPkexecProcess();

    let cmd = _config.PKEXEC_PATH + ' systemctl '+action+' '+_config.SERVICENAME_REDIS;
    if (numbersOfTryToActivateRedis == 0 && numbersOfTryToDesactivateRedis == 0) {
        try {
            Util.trySpawnCommandLine(cmd);
            statusIcon.set_property("style_class", armsLoading);
            if (action == "start") {
                GLib.timeout_add(0,300,tryActivateRedisService);
            } else {
                GLib.timeout_add(0,300,tryDesactivateRedisService);
            }
        } catch(Exception) {
            Main.notify("Big problem :/"+Exception);
        }
    }
}


function toggleSolr4Service() {
    let action = "start";
    if (isSolr4Active()) {
        action = "stop";
    }
    numbersOfPkexecProcess = getNumbersOfPkexecProcess();

    let cmd = _config.PKEXEC_PATH + ' systemctl '+action+' '+_config.SERVICENAME_SOLR4;
    if (numbersOfTryToActivateSolr4 == 0 && numbersOfTryToDesactivateSolr4 == 0) {
        try {
            Util.trySpawnCommandLine(cmd);
            statusIcon.set_property("style_class", armsLoading);
            if (action == "start") {
                GLib.timeout_add(0,300,tryActivateSolr4Service);
            } else {
                GLib.timeout_add(0,300,tryDesactivateSolr4Service);
            }
        } catch(Exception) {
            Main.notify("Big problem :/"+Exception);
        }
    }
}



function toggleSolr5Service() {
    let action = "start";
    if (isSolr5Active()) {
        action = "stop";
    }
    numbersOfPkexecProcess = getNumbersOfPkexecProcess();

    let cmd = _config.PKEXEC_PATH + ' systemctl '+action+' '+_config.SERVICENAME_SOLR5;
    if (numbersOfTryToActivateSolr5 == 0 && numbersOfTryToDesactivateSolr5 == 0) {
        try {
            Util.trySpawnCommandLine(cmd);
            statusIcon.set_property("style_class", armsLoading);
            if (action == "start") {
                GLib.timeout_add(0,300,tryActivateSolr5Service);
            } else {
                GLib.timeout_add(0,300,tryDesactivateSolr5Service);
            }
        } catch(Exception) {
            Main.notify("Big problem :/"+Exception);
        }
    }
}


function tryActivateApacheService() {
    let serviceWaiting = true;
    // We want to activate Apache
    if (numbersOfTryToActivateApache >= maxNumbersOfTry || isApacheActive()) {
        numbersOfTryToActivateApache = 0;
        if (!isPkExecThreadActive()){
            // PkExec is open ! don't do anything stupid
            if (isApacheActive()) {
                Main.notify("Apache is now on");
            } else {
                Main.notify("We couldn't managed to activate apache");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToActivateApache++;
    }
    return serviceWaiting;
}


function tryDesactivateApacheService() {
    let serviceWaiting = true;
    // We want to desactivate Apache
    if (numbersOfTryToDesactivateApache >= maxNumbersOfTry || !isApacheActive()) {
        numbersOfTryToDesactivateApache = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (!isApacheActive()) {
                Main.notify("Apache is now off");
            } else {
                Main.notify("We couldn't managed to desactivate apache");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToDesactivateApache++;
    }
    return serviceWaiting;
}


function tryActivateMysqlService() {
    let serviceWaiting = true;
    // We want to activate Mysql
    if (numbersOfTryToActivateMysql >= maxNumbersOfTry || isMysqlActive()) {
        numbersOfTryToActivateMysql = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (isMysqlActive()) {
                Main.notify("Mysql is now on");
            } else {
                Main.notify("We couldn't managed to activate MySQL");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToActivateMysql++;
    }
    return serviceWaiting;
}


function tryDesactivateMysqlService() {
    let serviceWaiting = true;
    // We want to desactivate Mysql
    if (numbersOfTryToDesactivateMysql >= maxNumbersOfTry || !isMysqlActive()) {
        numbersOfTryToDesactivateMysql = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (!isMysqlActive()) {
                Main.notify("Mysql is now off");
            } else {
                Main.notify("We couldn't managed to desactivate MySQL");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToDesactivateMysql++;
    }
    return serviceWaiting;
}

function tryActivateRedisService() {
    let serviceWaiting = true;
    // We want to activate Redis
    if (numbersOfTryToActivateRedis >= maxNumbersOfTry || isRedisActive()) {
        numbersOfTryToActivateRedis = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (isRedisActive()) {
                Main.notify("Redis is now on");
            } else {
                Main.notify("We couldn't managed to activate Redis");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToActivateRedis++;
    }
    return serviceWaiting;
}


function tryDesactivateRedisService() {
    let serviceWaiting = true;
    // We want to desactivate Redis
    if (numbersOfTryToDesactivateRedis >= maxNumbersOfTry || !isRedisActive()) {
        numbersOfTryToDesactivateRedis = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (!isRedisActive()) {
                Main.notify("Redis is now off");
            } else {
                Main.notify("We couldn't managed to desactivate Redis");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToDesactivateRedis++;
    }
    return serviceWaiting;
}



function tryActivateSolr4Service() {
    let serviceWaiting = true;
    // We want to activate Solr4
    if (numbersOfTryToActivateSolr4 >= maxNumbersOfTry || isSolr4Active()) {
        numbersOfTryToActivateSolr4 = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (isSolr4Active()) {
                Main.notify("Solr4 is now on");
            } else {
                Main.notify("We couldn't managed to activate Solr4");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToActivateSolr4++;
    }
    return serviceWaiting;
}


function tryDesactivateSolr4Service() {
    let serviceWaiting = true;
    // We want to desactivate Solr4
    if (numbersOfTryToDesactivateSolr4 >= maxNumbersOfTry || !isSolr4Active()) {
        numbersOfTryToDesactivateSolr4 = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (!isSolr4Active()) {
                Main.notify("Solr4 is now off");
            } else {
                Main.notify("We couldn't managed to desactivate Solr4");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToDesactivateSolr4++;
    }
    return serviceWaiting;
}


function tryActivateSolr5Service() {
    let serviceWaiting = true;
    // We want to activate Solr5
    if (numbersOfTryToActivateSolr5 >= maxNumbersOfTry || isSolr5Active()) {
        numbersOfTryToActivateSolr5 = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (isSolr5Active()) {
                Main.notify("Solr5 is now on");
            } else {
                Main.notify("We couldn't managed to activate Solr5");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToActivateSolr5++;
    }
    return serviceWaiting;
}


function tryDesactivateSolr5Service() {
    let serviceWaiting = true;
    // We want to desactivate Solr5
    if (numbersOfTryToDesactivateSolr5 >= maxNumbersOfTry || !isSolr5Active()) {
        numbersOfTryToDesactivateSolr5 = 0;
        if (!isPkExecThreadActive()){
            // PkExec is closed open ! don't do anything stupid
            if (!isSolr5Active()) {
                Main.notify("Solr5 is now off");
            } else {
                Main.notify("We couldn't managed to desactivate Solr5");
            }
            // No need to go to that loop again
            refreshUI();
            serviceWaiting = false;
        }
    } else {
        //it's not over !
        numbersOfTryToDesactivateSolr5++;
    }
    return serviceWaiting;
}

function getNumbersOfPkexecProcess() {
    // Get current status of mysql service
    let [resPkExec, outPkExec] = GLib.spawn_command_line_sync("pgrep pkexec -c");
    let outPkExecString = outPkExec.toString().replace(/(\r\n|\n|\r)/gm,"").trim();
    return outPkExecString;
}

function isPkExecThreadActive() {
    res = true;
    if (numbersOfPkexecProcess == getNumbersOfPkexecProcess()) {
        // The PkExec asking for passowrd is no longer active
        res = false;
    }
    return res;

}


function refreshUI() {
    refreshStatusIcon();
    refreshSwitchButton();
}

function refreshStatusIcon() {
    let activeServices = 0
    let icon_classname = armsStatusIcon[activeServices];

    if (isApacheActive()) {
        activeServices++;
    }
    if (isMysqlActive()) {
        activeServices++;
    }
    if (isRedisActive()) {
        activeServices++;
    }
    if (isSolr4Active()) {
        activeServices++;
    }
    if (isSolr5Active()) {
        activeServices++;
    }

    icon_classname = armsStatusIcon[activeServices];
    statusIcon.set_property("style_class", icon_classname);
}

function refreshSwitchButton() {
    menuItemApache.setToggleState(isApacheActive());
    menuItemMysql.setToggleState(isMysqlActive());
    menuItemRedis.setToggleState(isRedisActive());
    menuItemSolr4.setToggleState(isSolr4Active());
    menuItemSolr5.setToggleState(isSolr5Active());
}

